from django.core.management import BaseCommand, CommandError
import mysql.connector

from ippidb.models import Bibliography, Protein, Taxonomy, MolecularFunction, \
    Domain, ProteinDomainBoundComplex, ProteinDomainPartnerComplex, Symmetry, Ppi, PpiComplex

class Command(BaseCommand):

    help = "Import data from the local v1 database"

    def add_arguments(self, parser):
        parser.add_argument(
            '--bibliographies',
            action='store_true',
            dest='bibliographies',
            default=False,
            help='Flush and migrate bibliographies',
        )
        parser.add_argument(
            '--proteins',
            action='store_true',
            dest='proteins',
            default=False,
            help='Flush and migrate proteins',
        )
        parser.add_argument(
            '--domains',
            action='store_true',
            dest='domains',
            default=False,
            help='Flush and migrate domains',
        )
        parser.add_argument(
            '--symmetries',
            action='store_true',
            dest='symmetries',
            default=False,
            help='Flush and create symmetries',
        )
        parser.add_argument(
            '--ppi',
            action='store_true',
            dest='ppi',
            default=False,
            help='Flush and migrate ppis and complexes',
        )
        parser.add_argument(
            '--stoponfail',
            action='store_true',
            dest='stoponfail',
            default=False,
            help='Stop on fail',
        )


    def handle(self, *args, **options):
        conn = mysql.connector.connect(host="localhost",user="root",password="ippidb", database="ippidb")
        cursor = conn.cursor()
        if options['bibliographies']:
            cursor.execute("""SELECT * FROM biblio""")
            rows = cursor.fetchall()
            Bibliography.objects.all().delete()
            self.stdout.write(self.style.SUCCESS('Successfully flushed bibliography table'))
            for row in rows:
                try:
                    b = Bibliography()
                    if row[1]=='article':
                        b.source = 'PM'
                    else:
                        b.source = 'PT'
                    b.id_source = row[2]
                    b.save()
                except Exception as e:
                    self.stdout.write(self.style.ERROR('Failed inserting {}'.format(row[2])))
                else:
                    self.stdout.write(self.style.SUCCESS('Successfully inserted {}'.format(row[2])))
        if options['proteins']:
            cursor.execute("""SELECT * FROM protein""")
            rows = cursor.fetchall()
            Protein.objects.all().delete()
            self.stdout.write(self.style.SUCCESS('Successfully flushed protein table'))
            Taxonomy.objects.all().delete()
            self.stdout.write(self.style.SUCCESS('Successfully flushed taxonomy table'))
            MolecularFunction.objects.all().delete()
            self.stdout.write(self.style.SUCCESS('Successfully flushed molecular function table'))
            for row in rows:
                try:
                    p = Protein()
                    p.uniprot_id = row[1]
                    p.save()
                except Exception as e:
                    if options['stoponfail']:
                        import traceback
                        self.stderr.write(traceback.format_exc())
                        raise CommandError('Failed inserting {} {}'.format(row[1], row[2]))
                    else:
                        self.stdout.write(self.style.ERROR('Failed inserting {} {}'.format(row[1], row[2])))
                else:
                    self.stdout.write(self.style.SUCCESS('Successfully inserted {} {}'.format(row[1], row[2])))
        if options['domains']:
            cursor.execute("""SELECT * FROM domain""")
            rows = cursor.fetchall()
            Domain.objects.all().delete()
            self.stdout.write(self.style.SUCCESS('Successfully flushed domain table'))
            for row in rows:
                try:
                    p = Domain()
                    p.pfam_acc = row[2]
                    p.domain_family = row[4]
                    p.save()
                except Exception as e:
                    if options['stoponfail']:
                        import traceback
                        self.stderr.write(traceback.format_exc())
                        raise CommandError('Failed inserting {} {}'.format(row[1], row[2]))
                    else:
                        self.stdout.write(self.style.ERROR('Failed inserting {} {}'.format(row[1], row[2])))
                else:
                    self.stdout.write(self.style.SUCCESS('Successfully inserted {} {}'.format(row[1], row[2])))
        if options['symmetries']:
            Symmetry.objects.all().delete()
            self.stdout.write(self.style.SUCCESS('Successfully flushed symmetries table'))
            rows = [
                    ['AS', 'asymmetric'],
                    ['C2', 'C2 symmetry'],
                    ['D2', 'D2 symmetry'],
                    ['C3', 'C3 symmetry'],
                    ['D3', 'D3 symmetry'],
                    ['C4', 'C4 symmetry'],
                    ['D4', 'D4 symmetry'],
                    ['C5', 'C5 symmetry'],
                    ['D5', 'D5 symmetry'],
            ]
            for row in rows:
                try:
                    symmetry = Symmetry()
                    symmetry.code = row[0]
                    symmetry.description = row[1]
                    symmetry.save()
                except Exception as e:
                    if options['stoponfail']:
                        import traceback
                        self.stderr.write(traceback.format_exc())
                        raise CommandError('Failed inserting {} {}'.format(row[0], row[1]))
                    else:
                        self.stdout.write(self.style.ERROR('Failed inserting {} {}'.format(row[0], row[1])))
                else:
                    self.stdout.write(self.style.SUCCESS('Successfully inserted {} {}'.format(row[0], row[1])))
        if options['ppi']:
            sql_request_string = '''
select distinct protein.NumUniprot, domain.PfamNumAccession, complexe.NbCopy, cmpdAction.IDComplexeBound, bindingSiteEvidence.CodePDB, 'part1', ppi.IDPPI  from bindingSite inner join ppi on (bindingSite.IDBindingSite=ppi.IDBindingSite1) inner join complexe on (ppi.IDComplexe1=complexe.IDComplexe) left outer join cmpdAction on (complexe.IDComplexe=cmpdAction.IDComplexeBound) inner join protein on (bindingSite.IDProtein=protein.IDProtein) inner join domain on (bindingSite.IDDomain=domain.IDDomain) left outer join bindingSiteEvidence on (ppi.IDPPI=bindingSiteEvidence.IDPPI)
union 
select distinct protein.NumUniprot, domain.PfamNumAccession  , complexe.NbCopy, cmpdAction.IDComplexeBound, null, 'part2', ppi.IDPPI  from bindingSite inner join ppi on (bindingSite.IDBindingSite=ppi.IDBindingSite2) inner join complexe on (ppi.IDComplexe2=complexe.IDComplexe) left outer join cmpdAction on (complexe.IDComplexe=cmpdAction.IDComplexeBound) inner join protein on (bindingSite.IDProtein=protein.IDProtein) inner join domain on (bindingSite.IDDomain=domain.IDDomain)'''
            cursor.execute(sql_request_string)
            rows = cursor.fetchall()
            ProteinDomainBoundComplex.objects.all().delete()
            ProteinDomainPartnerComplex.objects.all().delete()
            Ppi.objects.all().delete()
            PpiComplex.objects.all().delete()
            self.stdout.write(self.style.SUCCESS('Successfully flushed protein domain complex tables and Ppi'))
            ppi_ids_mapping = {}
            for row in rows:
                try:
                    # create or retrieve Ppi object
                    if row[5]=='part1':
                        ppi = Ppi()
                        ppi.pdb_id = row[4]
                        ppi.symmetry = Symmetry.objects.get(code='AS')
                        ppi.save()
                        ppi_ids_mapping[row[6]]=ppi.id
                    else:
                        ppi = Ppi.objects.get(id=ppi_ids_mapping[row[6]])
                    # create a complex
                    if row[3] is None:
                        c = ProteinDomainPartnerComplex()
                    else:
                        c = ProteinDomainBoundComplex()
                    protein = Protein.objects.get(uniprot_id=row[0])
                    c.protein = protein
                    domain = Domain.objects.get(pfam_acc=row[1])
                    c.domain = domain
                    c.ppc_copy_nb = row[2]
                    if isinstance(c, ProteinDomainBoundComplex):
                        c.ppp_copy_nb_per_p = 1
                        c.pockets_nb = 1
                    c.save()
                    # create the PpiComplex object
                    ppi_complex = PpiComplex()
                    ppi_complex.ppi = ppi
                    ppi_complex.complex = c
                    ppi_complex.cc_nb = 1
                    ppi_complex.save()
                except Exception as e:
                    if options['stoponfail']:
                        import traceback
                        self.stderr.write(traceback.format_exc())
                        raise CommandError('Failed inserting {} {}'.format(row[0], row[1]))
                    else:
                        self.stdout.write(self.style.ERROR('Failed inserting {} {}'.format(row[0], row[1])))
                else:
                    self.stdout.write(self.style.SUCCESS('Successfully inserted {} {}'.format(row[0], row[1])))
            
